from utility.requesting import SyncRetryClient
import  requests

class GatewayHandler:
    def __init__(self, merchant_id, base_url: str):
        self.merchant_id = merchant_id
        self.base_url = base_url.rstrip('/')
        self.request = SyncRetryClient()

    def paylink(self, amount:int, callback_url: str, mobile: str, email: str, description=None)->requests.Response:
        
        '''
        Create a paylink
        :param amount: Amount in cents
        :param callback_url: Callback URL
        :param mobile: Mobile number
        :param email: Email address
        :param description: Description
        :return json:
            {
                    'status': True,
                    'data': {
                        'payment_url': payment_url,
                        'token': transaction.user_token,
                        'amount': float(transaction.amount),
                    },
                    'error': None
                }
        
        '''
        url = f'{self.base_url}/create/'
        headers = {
            'Authorization': f'Token {self.merchant_id}'
        }
        data = {
            'amount': amount,
            'callback_url': callback_url,
            'phone': mobile,
            'email': email,
            'description': description
        }
        return self.request.post(url, data, headers=headers)

    def verify(self, amount: int, token: str)->requests.Response:
        '''
        Verify a transaction
        :param amount: Amount in cents
        :param token: Transaction token
        :return json:
            {
                    'status': True,
                    'amount': float,
                    'error': None
                }
        '''
        url = f'{self.base_url}/verify/'
        headers = {
            'Authorization': f'Token {self.merchant_id}'
        }

        data = {
            'amount': amount,
            'token': token,
        }
        return self.request.post(url, data, headers=headers)




import time
from functools import wraps

def timing_decorator(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        end_time = time.time()
        execution_time = end_time - start_time
        print(f"Function '{func.__name__}' took {execution_time:.4f} seconds to execute")
        return result
    return wrapper



def chunks(input_data, sizes:int|list|tuple, reverse=False,append_last=True):
    """
    Splits a list or dictionary into sublists/subdicts based on the specified sizes.

    :param input_data: The list or dictionary to be split
    :param sizes: A tuple or list of integers representing the sizes of the sublists/subdicts
    :param reverse: If True, reverses each chunk
    :param append_last: If True, appends remaining elements as last chunk
    :return: A list of sublists/subdicts split according to the sizes
    """
    do_reverse = lambda x: x[::-1] if reverse else x
    
    # Handle dictionary input
    if isinstance(input_data, dict):
        items = list(input_data.items())
        chunks_result = chunks(items, sizes, reverse, append_last)
        return [dict(chunk) for chunk in chunks_result]
    
    # Original list handling
    if isinstance(sizes, int):
        res = []
        for i in range(0, len(input_data), sizes):
            res += [do_reverse(input_data[i:i + sizes])]
        return res
        
    result = []
    start_index = 0

    for size in sizes:
        end_index = start_index + size
        result.append(do_reverse(input_data[start_index:end_index]))
        start_index = end_index
    if append_last and start_index < len(input_data):
        result.append(do_reverse(input_data[start_index:]))

    return result


import collections
def sort_dict_by_key(item:dict,reverse=False):
    return dict(collections.OrderedDict(sorted(item.items(),reverse=reverse)))

def sort_dict_by_val(item:dict,reverse=False):
    sorted_x = sorted(item.items(), key=lambda kv: kv[1],reverse=reverse)
    return dict(collections.OrderedDict(sorted_x))



import pytest
import logging
import os
from utility.logger import Logger
from unittest.mock import patch, MagicMock

@pytest.fixture
def test_logger():
    return Logger("test_logger", alert=False)

@pytest.fixture
def alert_logger():
    return Logger("alert_logger", alert=True)

def test_logger_initialization(test_logger):
    assert isinstance(test_logger, Logger)
    assert test_logger.name == "test_logger"
    assert test_logger.level == logging.DEBUG
    assert test_logger.alert is False
    assert len(test_logger.handlers) == 2  # File and console handlers

def test_logger_file_creation(test_logger):
    log_file = f"{test_logger.name}.log"
    assert os.path.exists(log_file)
    os.remove(log_file)  # Cleanup

def test_logging_levels(test_logger):
    with patch.object(test_logger, '_log_with_telegram') as mock_log:
        test_logger.debug("Debug message")
        mock_log.assert_called_with(logging.ERROR, "Debug message", exc_info=True)
        
        test_logger.info("Info message")
        mock_log.assert_called_with(logging.INFO, "Info message")
        
        test_logger.warning("Warning message")
        mock_log.assert_called_with(logging.WARNING, "Warning message")
        

def test_telegram_alert_disabled(test_logger):
    result = test_logger.send_telegram_message("Test message")
    assert result is False

@patch('requests.post')
def test_telegram_alert_enabled(mock_post, alert_logger):
    mock_response = MagicMock()
    mock_response.status_code = 200
    mock_post.return_value = mock_response
    
    result = alert_logger.send_telegram_message("Test message")
    assert result is True
    mock_post.assert_called_once()

def test_alert_level_include():
    logger = Logger("test", alert=True, alert_log_level_inclue=[logging.ERROR])
    with patch.object(logger, 'send_telegram_message') as mock_send:
        logger.info("This shouldn't trigger alert")
        mock_send.assert_not_called()
        
        logger.error("This should trigger alert")
        mock_send.assert_called_once()

def test_alert_level_exclude():
    logger = Logger("test", alert=True, alert_log_level_exclude=[logging.INFO])
    with patch.object(logger, 'send_telegram_message') as mock_send:
        logger.info("This shouldn't trigger alert")
        mock_send.assert_not_called()
        
        logger.error("This should trigger alert")
        mock_send.assert_called_once()

def test_exception_logging(test_logger):
    with patch.object(test_logger, '_log_with_telegram') as mock_log:
        try:
            raise ValueError("Test exception")
        except ValueError:
            test_logger.error("Error occurred")
            mock_log.assert_called_once()
            args, kwargs = mock_log.call_args
            assert kwargs.get('exc_info') is True



import os
import pytest
from utility.file_dir import FileFinder

@pytest.fixture
def temp_directory(tmp_path):
    # Create test files and directories
    test_files = [
        'test1.txt',
        'test2.txt',
        'test3.pdf',
        'subdir/test4.txt',
        'subdir/test5.pdf'
    ]
    
    for file_path in test_files:
        full_path = tmp_path / file_path
        full_path.parent.mkdir(parents=True, exist_ok=True)
        full_path.touch()
    
    return tmp_path

def test_find_all_files(temp_directory):
    finder = FileFinder(temp_directory)
    files = finder.find_all_files()
    
    # Should find all 5 files
    assert len(files) == 5
    
    # Convert paths to set of basenames for easier comparison
    file_names = {os.path.basename(f) for f in files}
    expected_names = {'test1.txt', 'test2.txt', 'test3.pdf', 'test4.txt', 'test5.pdf'}
    assert file_names == expected_names

def test_find_by_extension(temp_directory):
    finder = FileFinder(temp_directory)
    
    # Test .txt files
    txt_files = finder.find_by_extension('.txt')
    assert len(txt_files) == 3
    txt_names = {os.path.basename(f) for f in txt_files}
    assert txt_names == {'test1.txt', 'test2.txt', 'test4.txt'}
    
    # Test .pdf files
    pdf_files = finder.find_by_extension('.pdf')
    assert len(pdf_files) == 2
    pdf_names = {os.path.basename(f) for f in pdf_files}
    assert pdf_names == {'test3.pdf', 'test5.pdf'}

def test_get_file_count(temp_directory):
    finder = FileFinder(temp_directory)
    assert finder.get_file_count() == 5

def test_empty_directory(tmp_path):
    finder = FileFinder(tmp_path)
    assert finder.get_file_count() == 0
    assert len(finder.find_all_files()) == 0
    assert len(finder.find_by_extension('.txt')) == 0



from deep_translator import GoogleTranslator
def translator(txt,dest,src='auto'):
    if dest=='ch':dest = 'zh-CN'
    try:return GoogleTranslator(source=src, target=dest).translate(txt)
    except Exception as e:print(e)

import random

def fake_mobile():
    # Valid Iranian mobile prefixes
    prefixes = ['0910', '0911', '0912', '0913', '0914', '0915', '0916', '0917', 
                '0918', '0919', '0990', '0991', '0992', '0993', '0994']
    
    # Select random prefix
    prefix = random.choice(prefixes)
    
    # Generate random 7 digits
    suffix = ''.join([str(random.randint(0, 9)) for _ in range(7)])
    
    # Combine prefix and suffix
    phone_number = prefix + suffix
    
    return phone_number

import random

def fake_email():
    # Iranian names and surnames
    names = ['علی', 'محمد', 'حسین', 'رضا', 'امیر', 'مهدی', 'سارا', 'مریم', 'فاطمه', 'زهرا',
             'امین', 'حسن', 'نیما', 'پویا', 'سینا', 'آرش', 'شیما', 'نازنین', 'پریسا', 'لیلا']
    
    surnames = ['محمدی', 'حسینی', 'رضایی', 'کریمی', 'موسوی', 'هاشمی', 'احمدی', 'اکبری', 'علوی', 'رحیمی',
                'صادقی', 'نجفی', 'عباسی', 'حیدری', 'قاسمی', 'طاهری', 'نوری', 'یوسفی', 'مرادی', 'جعفری']
    
    domains = ['gmail.com', 'yahoo.com', 'hotmail.com', 'outlook.com']
    
    # Generate unique combinations
    name = random.choice(names)
    surname = random.choice(surnames)
    domain = random.choice(domains)
    
    # Add random number for uniqueness (1-999)
    random_number = random.randint(65, 80)
    
    # Convert Persian names to English transliteration
    name_en = {
        'علی': 'ali', 'محمد': 'mohammad', 'حسین': 'hossein', 'رضا': 'reza',
        'امیر': 'amir', 'مهدی': 'mehdi', 'سارا': 'sara', 'مریم': 'maryam',
        'فاطمه': 'fateme', 'زهرا': 'zahra', 'امین': 'amin', 'حسن': 'hasan',
        'نیما': 'nima', 'پویا': 'pouya', 'سینا': 'sina', 'آرش': 'arash',
        'شیما': 'shima', 'نازنین': 'nazanin', 'پریسا': 'parisa', 'لیلا': 'leila'
    }
    
    surname_en = {
        'محمدی': 'mohammadi', 'حسینی': 'hosseini', 'رضایی': 'rezaei', 'کریمی': 'karimi',
        'موسوی': 'mousavi', 'هاشمی': 'hashemi', 'احمدی': 'ahmadi', 'اکبری': 'akbari',
        'علوی': 'alavi', 'رحیمی': 'rahimi', 'صادقی': 'sadeghi', 'نجفی': 'najafi',
        'عباسی': 'abbasi', 'حیدری': 'heidari', 'قاسمی': 'ghasemi', 'طاهری': 'taheri',
        'نوری': 'nouri', 'یوسفی': 'yousefi', 'مرادی': 'moradi', 'جعفری': 'jafari'
    }
    
    # Create email formats
    formats = [
        f"{name_en[name]}{surname_en[surname]}{random_number}@{domain}",
        f"{name_en[name]}.{surname_en[surname]}{random_number}@{domain}",
        f"{surname_en[surname]}.{name_en[name]}{random_number}@{domain}",
        f"{name_en[name]}_{surname_en[surname]}{random_number}@{domain}"
        f"{name_en[name]}{surname_en[surname]}@{domain}",
        f"{name_en[name]}.{surname_en[surname]}@{domain}",
        f"{surname_en[surname]}.{name_en[name]}@{domain}",
        f"{name_en[name]}_{surname_en[surname]}@{domain}"
    ]
    
    if random.choice([True, False]):
        nickname = generate_nickname()
        formats.extend([
            f"{name_en[name]}{surname_en[surname]}.{nickname}@{domain}",
            f"{name_en[name]}.{surname_en[surname]}@{domain}",
            f"{surname_en[surname]}.{nickname}.{name_en[name]}@{domain}",
            f"{name_en[name]}_{surname_en[surname]}_{nickname}@{domain}"
        ])
    
    return random.choice(formats)

def generate_nickname():
    
    nouns = ['tech', 'pro','wolf', 'dragon', 'hunter','king', 'shadow']
    
    
    return random.choice(nouns).lower()



import pytest
import aiohttp
import responses
from utility.requesting import SyncRetryClient, AsyncRetryClient

# Fixtures
@pytest.fixture
def sync_client():
    return SyncRetryClient(retries=2, backoff_factor=0.1)

@pytest.fixture
def async_client():
    return AsyncRetryClient(retries=2, delay=0.1)

# Sync Client Tests
@responses.activate
def test_sync_get_success(sync_client):
    responses.add(
        responses.GET,
        'https://api.example.com/test',
        json={'status': 'success'},
        status=200
    )
    
    response = sync_client.get('https://api.example.com/test')
    assert response.status_code == 200
    assert response.json() == {'status': 'success'}

@responses.activate
def test_sync_post_success(sync_client):
    responses.add(
        responses.POST,
        'https://api.example.com/test',
        json={'status': 'created'},
        status=201
    )
    
    response = sync_client.post(
        'https://api.example.com/test',
        json={'data': 'test'}
    )
    assert response.status_code == 201
    assert response.json() == {'status': 'created'}

@responses.activate
def test_sync_retry_on_failure(sync_client):
    # Add two failure responses and one success
    responses.add(
        responses.GET,
        'https://api.example.com/test',
        status=503
    )
    responses.add(
        responses.GET,
        'https://api.example.com/test',
        json={'status': 'success'},
        status=200
    )
    
    response = sync_client.get('https://api.example.com/test')
    assert response.status_code == 200
    assert len(responses.calls) > 1

# Async Client Tests
@pytest.mark.asyncio
async def test_async_get_success(async_client, aiohttp_client):
    async def mock_server(request):
        return aiohttp.web.json_response({'status': 'success'})

    app = aiohttp.web.Application()
    app.router.add_get('/test', mock_server)
    client = await aiohttp_client(app)
    
    response = await async_client.get(f'{client.make_url("/test")}')
    assert 'success' in response

@pytest.mark.asyncio
async def test_async_post_success(async_client, aiohttp_client):
    async def mock_server(request):
        return aiohttp.web.json_response({'status': 'created'}, status=201)

    app = aiohttp.web.Application()
    app.router.add_post('/test', mock_server)
    client = await aiohttp_client(app)
    
    response = await async_client.post(
        f'{client.make_url("/test")}',
        json={'data': 'test'}
    )
    assert 'created' in response

@pytest.mark.asyncio
async def test_async_retry_on_failure(async_client, aiohttp_client):
    call_count = 0
    
    async def mock_server(request):
        nonlocal call_count
        call_count += 1
        if call_count < 2:
            return aiohttp.web.Response(status=503)
        return aiohttp.web.json_response({'status': 'success'})

    app = aiohttp.web.Application()
    app.router.add_get('/test', mock_server)
    client = await aiohttp_client(app)
    
    response = await async_client.get(f'{client.make_url("/test")}')
    assert 'success' in response
    assert call_count > 1



import pytest
import os
import tempfile
from pathlib import Path
from utility.file_dir import ZipUtility

class TestZipUtility:
    @pytest.fixture
    def setup_test_env(self):
        # Create temporary directory for tests
        with tempfile.TemporaryDirectory() as temp_dir:
            # Create test files and directories
            test_dir = Path(temp_dir)
            
            # Create test files
            (test_dir / "file1.txt").write_text("Test content 1")
            (test_dir / "file2.txt").write_text("Test content 2")
            
            # Create nested directory with files
            nested_dir = test_dir / "nested"
            nested_dir.mkdir()
            (nested_dir / "nested_file.txt").write_text("Nested content")
            
            yield test_dir
    
    @pytest.fixture
    def zip_utility(self):
        return ZipUtility()
    
    def test_zip_single_file(self, setup_test_env, zip_utility):
        test_dir = setup_test_env
        source_file = test_dir / "file1.txt"
        zip_name = test_dir / "single_file.zip"
        
        # Test zip creation
        result = zip_utility.zip_files(str(source_file), str(zip_name))
        assert result == True
        assert zip_name.exists()
        
        # Verify zip contents
        info = zip_utility.get_zip_info(str(zip_name))
        assert info['file_count'] == 1
        assert info['files'][0]['filename'] == "file1.txt"
    
    def test_zip_multiple_files(self, setup_test_env, zip_utility):
        test_dir = setup_test_env
        source_files = [
            str(test_dir / "file1.txt"),
            str(test_dir / "file2.txt")
        ]
        zip_name = test_dir / "multiple_files.zip"
        
        result = zip_utility.zip_files(source_files, str(zip_name))
        assert result == True
        
        info = zip_utility.get_zip_info(str(zip_name))
        assert info['file_count'] == 2
    
    def test_zip_directory(self, setup_test_env, zip_utility):
        test_dir = setup_test_env
        zip_name = test_dir / "directory.zip"
        
        result = zip_utility.zip_files(str(test_dir / "nested"), str(zip_name))
        assert result == True
        
        info = zip_utility.get_zip_info(str(zip_name))
        assert info['file_count'] == 1
        assert any("nested_file.txt" in file['filename'] for file in info['files'])
    
    def test_unzip_files(self, setup_test_env, zip_utility):
        test_dir = setup_test_env
        source_file = test_dir / "file1.txt"
        zip_name = test_dir / "test_unzip.zip"
        extract_dir = test_dir / "extracted"
        
        # Create zip first
        zip_utility.zip_files(str(source_file), str(zip_name))
        
        # Test extraction
        result = zip_utility.unzip_files(str(zip_name), str(extract_dir))
        assert result == True
        assert (extract_dir / "file1.txt").exists()
        assert (extract_dir / "file1.txt").read_text() == "Test content 1"
    
    def test_get_zip_info(self, setup_test_env, zip_utility):
        test_dir = setup_test_env
        source_file = test_dir / "file1.txt"
        zip_name = test_dir / "info_test.zip"
        
        zip_utility.zip_files(str(source_file), str(zip_name))
        
        info = zip_utility.get_zip_info(str(zip_name))
        assert info is not None
        assert 'file_count' in info
        assert 'total_size' in info
        assert 'compressed_size' in info
        assert 'files' in info
        assert len(info['files']) == 1
    
    def test_invalid_zip_operations(self, setup_test_env, zip_utility):
        test_dir = setup_test_env
        
        # Test non-existent source
        result = zip_utility.zip_files("nonexistent.txt", "test.zip")
        assert result == False
        
        # Test invalid zip file for extraction
        result = zip_utility.unzip_files("nonexistent.zip")
        assert result == False
        
        # Test invalid zip file for info
        info = zip_utility.get_zip_info("nonexistent.zip")
        assert info is None



import logging
from logging.handlers import RotatingFileHandler
import requests
import os,sys
from .file_dir import root_path,directory_creator

class Logger(logging.Logger):
    def __init__(self, name: str, level: int = logging.DEBUG, bot_token: str='7642891094:AAFaKkfCPJQlWyP9orrjaAN9nOz9a5tl6Wc', chat_id: str='1119223961', alert:bool=False,pov:str=None,alert_log_level_inclue:list=None,alert_log_level_exclude:list=None,**kwargs) -> None:
        '''
        log_dir:str=''
        '''
        log_dir=root_path()+'/'+kwargs.get('log_dir',f"/").rstrip('/')
        super().__init__(name, level)
        log_file=f"{log_dir}/{name}.log"
        self.bot_token = bot_token
        self.chat_id = chat_id
        self.alert = alert
         
        self.pov=pov
        self.alert_log_level_inclue=alert_log_level_inclue
        self.alert_log_level_exclude=alert_log_level_exclude
        
        self._formatter = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        self._telegram_formatter = kwargs.get('telegram_formatter',"%(name)s - %(levelname)s - %(message)s")
        # Avoid adding duplicate handlers
        if not self.hasHandlers():
            # File handler with rotation
            file_handler = RotatingFileHandler(
                log_file, maxBytes=5 * 1024 * 1024, backupCount=5
            )
            file_handler.setLevel(level)

            # Console handler
            console_handler = logging.StreamHandler()
            console_handler.setLevel(logging.INFO)

            # Formatter for handlers
            formatter = logging.Formatter(
                self._formatter
            )
            file_handler.setFormatter(formatter)
            console_handler.setFormatter(formatter)

            # Add handlers to the logger
            self.addHandler(file_handler)
            self.addHandler(console_handler)

    def send_telegram_message(self, message: str) -> bool:
        """Send message to Telegram if alerting is enabled and credentials are set"""
        if not all([self.alert, self.bot_token, self.chat_id]):
            return False
            
        try:
            url = f"https://api.telegram.org/bot{self.bot_token}/sendMessage"
            data = {
                "chat_id": self.chat_id,
                "text": message,
                "parse_mode": "HTML"
            }
            response = requests.post(url, data=data)
            return response.status_code == 200
        except Exception as e:
            super().error(f"Failed to send Telegram alert: {str(e)}")
            return False

    def _log_with_telegram(self, level, msg, *args, **kwargs):
        """Internal method to handle logging with Telegram alert"""
        # First, perform normal logging
        # Get correct caller info
        try:
            pathname, lineno, _,_ = self.findCaller()
        except ValueError:
            pathname, lineno, _ = "(unknown file)", 0, "(unknown function)"
        if level not in (logging.ERROR, logging.CRITICAL):
            logMsg=f"(File \"{pathname}\", line {lineno}):\n=>{msg}\n"
        else:logMsg=msg
        super()._log(level, logMsg, args, **kwargs)
        if self.alert_log_level_inclue and level not in self.alert_log_level_inclue:
            return
        if self.alert_log_level_exclude and level in self.alert_log_level_exclude:
            return
        # Then send to Telegram if enabled
        if self.alert and self.bot_token and self.chat_id:
            exc_info = kwargs.get('exc_info', None)
            # Get the caller's frame
            
            
            record = logging.LogRecord(
                name=self.name,
                level=level,
                pathname=pathname,
                lineno=lineno,
                msg=msg,
                args=args,
                exc_info=None
            )
            formatted_msg = logging.Formatter(self._telegram_formatter).format(record)
            level_name = logging.getLevelName(level)
            pov = f"pov={self.pov}\n" if self.pov else ''
        
            # Add traceback for ERROR and CRITICAL levels
            if level in (logging.ERROR, logging.CRITICAL) and exc_info:
                import traceback
                if exc_info is True:
                    exc_info = sys.exc_info()
                if exc_info:
                    tb = ''.join(traceback.format_exception(*exc_info))
                    formatted_msg = f"{formatted_msg}\n\n<pre>{tb}</pre>"
        
            telegram_msg = f"<b>{level_name}</b>\n{pov}{formatted_msg}"
            self.send_telegram_message(telegram_msg)

    def debug(self, msg, *args, **kwargs):
        self._log_with_telegram(logging.DEBUG, msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        self._log_with_telegram(logging.INFO, msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        self._log_with_telegram(logging.WARNING, msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        kwargs.setdefault("exc_info", True)
        self._log_with_telegram(logging.ERROR, msg, *args, **kwargs)

    def critical(self, msg, *args, **kwargs):
        kwargs.setdefault("exc_info", True)
        self._log_with_telegram(logging.CRITICAL, msg, *args, **kwargs)
    def debug(self,e):
        self.error(e)
    